% =========================================================
% SK-Hype Pro algorithm
% ---------------------------------------------------------
% Related paper: 
% 
% J. CHEN, C. RICHARD and P. HONEINE, "Nonlinear unmixing of hyperspectral
% data based on a linear-mixture/nonlinear-fluctuation model". IEEE Transactions on signal processing, 2012. 
% 
% This is an improvement of version of Sk-Hype. The balance coefficients
% arre optimized directly instead of using the gradient.
% ---------------------------------------------------------
% Utilization recommendation:
% 1. Parameters (Kernel bandwidth/regularization parameter) should be tuned according to scenes.
% 2. Here we use the function "qpas" for the optimization instead of the general optimization 
%    function of matlab: "quadprog", as the later is rather slow. One can
%    replace all "qpas" by "quadprog", or other optimization functions in case
%    there's a problem of running.
%    The full toolbox QuadprogC is also attached for respecting its
%    completeness. 
%
% Version: 13-sept-2012
%
% Jie CHEN & Cedric Richard
% chen@unice.fr, cedric.richard@unice.fr
% Laboratoire Lagrange, Universit de Nice Sophia-antipolis
% Nice, France

clear


% ============  Parameters to tune ======================== 
% Gaussian kernel bandwidth : 
par = 2;
% Regualrization parameter : 
% \mu in the paper = 1/C
C = 100;


% ============== Scene Parameters =========================
%  (see the function generate_image for detailes)
% endmember database (only 3 availabe in this demo due to the endmembers.mat)
Material_database = 3;
% number of endmember (maximum to 8 in this demo due to the endmembers.mat)
R = 5; 
% mixture model (1: linear, 2: bilinear, 3:postnonlinear)
Mixture_model=3;
% number of pixels to generate
N = 10;
% snr
SNR = 30;

%  ============== Image generation  =======================
[r,M,a,noise_std]=generate_image(Material_database, R, Mixture_model, N, SNR);
[L, R] = size(M);



% ============= Gaussian kernel calculation =================
Q=eye(R)/par^2;
MQM=M*Q*M';
dMQM = diag(MQM);
KM = exp(-0.5*(dMQM*ones(L,1)'+ones(L,1)*dMQM'-2*M*Q*M'));
% For using the polynomial proposed polynomial kernel, remove the
% comment symbol:
% KM = (1+1/R^2*(M-0.5)*(M-0.5)').^2;


MM =M*M';




d = [0.5;0.5];

iter_total=0;
ttt=0;

v=0.0;
tic

for n = 1 : N
    if mod(n,10)==0, n, end
    y = r(:,n);
    d = [0.5;0.5];
    K = d(1)*MM+d(2)*KM;
    Kt = [K+1/C*eye(L)];
    H = [Kt,    d(1)*M;
         d(1)*M',  d(1)*eye(R)];        
    H=(H+H')/2+0e-5*eye(L+R);
    f = -[y;zeros(R,1)];
    A = -[zeros(R,L),eye(R)];
    b = zeros(R,1);    

    z0 = qpas(H,f,A,b);
    
    z = z0(1:L);
    gam = z0(L+1:L+R);
    
    ht = (M'*z+gam)*d(1);

    
    for iter = 1 : 15                        % Maximum iteration 15
 
        
        d(1) = 1/(1+sqrt((d(2)^2*z'*KM*z)/(ht'*ht)));
        d(2)=1-d(1);

        K = d(1)*MM+d(2)*KM;
        Kt = [K+1/C*eye(L)];
        H = [Kt,    d(1) *M;
            d(1) *M',  d(1) *eye(R)];        
        H=(H+H')/2+0e-5*eye(L+R);
        z0 = qpas(H,f,A,b); 
        z = z0(1:L);
        gam = z0(L+1:L+R); 
        ht = (M'*z+gam)*d(1); 
        if (iter >1 & norm(d-dp)/R< 1e-4)    % Stop criterion
             break;
        end
        dp=d;
        
    end


  %   h = d(1)*(M'*beta+gam);
     a_est(:,n)=ht/sum(ht);
      
end
toc
[RMSE, std] = ErrComput(a(:,1:N),a_est)


